import 'package:flutter/material.dart';
import 'package:meta/meta.dart';

class _NavButton extends StatelessWidget {
  final double position;
  final int length;
  final int index;
  final ValueChanged<int> onTap;
  final Widget child;

  _NavButton({this.onTap, this.position, this.length, this.index, this.child});

  @override
  Widget build(BuildContext context) {
    final desiredPosition = 1.0 / length * index;
    final difference = (position - desiredPosition).abs();
    final verticalAlignment = 1 - length * difference;
    final opacity = length * difference;
    return Expanded(
      child: InkWell(
        onTap: () {
          onTap(index);
        },
        child: Container(
            height: 75.0,
            child: Transform.translate(
              offset: Offset(
                  0, difference < 1.0 / length ? verticalAlignment * 40 : 0),
              child: Opacity(
                  opacity: difference < 1.0 / length * 0.99 ? opacity : 1.0,
                  child: child),
            )),
      ),
    );
  }
}

class NavCustomPainter extends CustomPainter {
  double loc;
  double s;
  Color color;
  Gradient gradient;

  NavCustomPainter(double startingLoc, int itemsLength, this.color,
      {this.gradient}) {
    final span = 1.0 / itemsLength;
    s = 0.2;
    loc = startingLoc + (span - s) / 2;
  }

  @override
  void paint(Canvas canvas, Size size) {
    if (this.gradient == null) {
      gradient = new LinearGradient(
        colors: <Color>[
          Colors.grey[900],
          Colors.blueGrey[900],
          Colors.grey[900],
        ],
        stops: <double>[
          loc - (s / 2),
          loc + (s / 2),
          loc + (s / 2 + s),
        ],
        begin: Alignment.centerLeft,
        end: Alignment.centerRight,
      );
    }

    final paint = Paint()
      ..color = color
      ..shader =
          gradient.createShader(Rect.fromLTWH(0, 0, size.width, size.height))
      ..isAntiAlias = true
      ..filterQuality = FilterQuality.high
      ..style = PaintingStyle.fill;

    final path = Path()
      ..moveTo(0, 0)
      ..lineTo((loc - 0.08) * size.width, 0)
      ..cubicTo(
        (loc + s * 0.24) * size.width,
        size.height * 0.005,
        loc * size.width,
        size.height * 0.75,
        (loc + s * 0.50) * size.width,
        size.height * 0.80,
      )
      ..cubicTo(
        (loc + s) * size.width,
        size.height * 0.75,
        (loc + s - s * 0.24) * size.width,
        size.height * 0.005,
        (loc + s + 0.08) * size.width,
        0,
      )
      ..lineTo(size.width, 0)
      ..lineTo(size.width, size.height)
      ..lineTo(0, size.height)
      ..close();
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return this != oldDelegate;
  }
}

class BottomNavigationBarXItem {
  final Widget icon;
  final Widget label;
  BottomNavigationBarXItem({
    this.icon,
    this.label,
  });
}

class BottomNavigationBarX extends StatefulWidget {
  final List<BottomNavigationBarXItem> items;
  final int initialIndex;
  final Color color;
  final Gradient gradient;
  final Color buttonBackgroundColor;
  final Color backgroundColor;
  final ValueChanged<int> onTap;
  final Curve animationCurve;
  final Duration animationDuration;

  BottomNavigationBarX(
      {Key key,
      @required this.items,
      this.initialIndex = 0,
      this.color = Colors.white,
      this.gradient,
      this.buttonBackgroundColor,
      this.backgroundColor = Colors.blueAccent,
      this.onTap,
      this.animationCurve = Curves.easeOut,
      this.animationDuration = const Duration(milliseconds: 500)})
      : assert(items != null),
        assert(items.length >= 2),
        assert(0 <= initialIndex && initialIndex < items.length),
        super(key: key);

  @override
  _BottomNavigationBarXState createState() => _BottomNavigationBarXState();
}

class _BottomNavigationBarXState extends State<BottomNavigationBarX>
    with SingleTickerProviderStateMixin {
  double _startingPos;
  int _endingIndex = 0;
  double _pos;
  double _buttonHide = 0;
  Widget _icon;
  AnimationController _animationController;
  int _length;

  @override
  void initState() {
    super.initState();
    _icon = widget.items[widget.initialIndex].icon;
    _length = widget.items.length;
    _pos = widget.initialIndex / _length;
    _startingPos = widget.initialIndex / _length;
    _animationController = AnimationController(vsync: this, value: _pos);
    _animationController.addListener(() {
      setState(() {
        _pos = _animationController.value;
        final endingPos = _endingIndex / widget.items.length;
        final middle = (endingPos + _startingPos) / 2;
        if ((endingPos - _pos).abs() < (_startingPos - _pos).abs()) {
          _icon = widget.items[_endingIndex].icon;
        }
        _buttonHide =
            (1 - ((middle - _pos) / (_startingPos - middle)).abs()).abs();
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    Size size = MediaQuery.of(context).size;
    return Container(
      color: widget.backgroundColor,
      child: Padding(
        padding: const EdgeInsets.only(top: 5.0),
        child: Container(
          color: widget.backgroundColor,
          height: 60.0,
          child: Stack(
            overflow: Overflow.visible,
            alignment: Alignment.bottomCenter,
            children: <Widget>[
              Positioned(
                bottom: -60,
                left: _pos * size.width,
                width: size.width / _length,
                child: Center(
                  child: Transform.translate(
                    offset: Offset(
                      0,
                      -(1 - _buttonHide) * 80,
                    ),
                    child: Material(
                      shadowColor: Colors.black,
                      elevation: 2,
                      color: widget.buttonBackgroundColor ?? widget.color,
                      borderOnForeground: true,
                      type: MaterialType.circle,
                      child: Padding(
                        padding: const EdgeInsets.all(18),
                        child: _icon,
                      ),
                    ),
                  ),
                ),
              ),
              Align(
                alignment: Alignment.bottomCenter,
                child: CustomPaint(
                  painter: NavCustomPainter(_pos, _length, widget.color,
                      gradient: widget.gradient),
                  child: Container(
                    height: 60.0,
                  ),
                ),
              ),
              Align(
                alignment: Alignment.bottomCenter,
                child: SizedBox(
                  height: 60.0,
                  child: Row(
                    children: widget.items.map((item) {
                      return _NavButton(
                        onTap: _buttonTap,
                        position: _pos,
                        length: _length,
                        index: widget.items.indexOf(item),
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.center,
                          crossAxisAlignment: CrossAxisAlignment.center,
                          mainAxisSize: MainAxisSize.min,
                          children: <Widget>[
                            item.icon,
                            item.label != null
                                ? Container(
                                    height: 3,
                                  )
                                : Container(),
                            item.label != null ? item.label : Container(),
                          ],
                        ),
                      );
                    }).toList(),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  void _buttonTap(int index) {
    if (widget.onTap != null) {
      widget.onTap(index);
    }
    final newPosition = index / _length;
    setState(() {
      _startingPos = _pos;
      _endingIndex = index;
      _animationController.animateTo(newPosition,
          duration: widget.animationDuration, curve: widget.animationCurve);
    });
  }
}
